
"""
    show(io::IO, process::QEDProcessDescription)

Pretty print an [`QEDProcessDescription`](@ref) (no newlines).

```jldoctest
julia> using MetagraphOptimization

julia> print(parse_process("ke->ke", QEDModel()))
QED Process: 'ke->ke'

julia> print(parse_process("kk->ep", QEDModel()))
QED Process: 'kk->ep'
```
"""
function show(io::IO, process::QEDProcessDescription)
    # types() gives the types in order (QED) instead of random like keys() would
    print(io, "QED Process: \'")
    for type in types(QEDModel())
        for _ in 1:get(process.inParticles, type, 0)
            print(io, String(type))
        end
    end
    print(io, "->")
    for type in types(QEDModel())
        for _ in 1:get(process.outParticles, type, 0)
            print(io, String(type))
        end
    end
    print(io, "'")
    return nothing
end

"""
    show(io::IO, processInput::QEDProcessInput)

Pretty print an [`QEDProcessInput`](@ref) (with newlines).
"""
function show(io::IO, processInput::QEDProcessInput)
    println(io, "Input for $(processInput.process):")
    println(io, "  $(length(processInput.inParticles)) Incoming particles:")
    for particle in processInput.inParticles
        println(io, "    $particle")
    end
    println(io, "  $(length(processInput.outParticles)) Outgoing Particles:")
    for particle in processInput.outParticles
        println(io, "    $particle")
    end
    return nothing
end

"""
    show(io::IO, particle::T) where {T <: QEDParticle}

Pretty print an [`QEDParticle`](@ref) (no newlines).
"""
function show(io::IO, particle::T) where {T <: QEDParticle}
    print(io, "$(String(typeof(particle))): $(particle.momentum)")
    return nothing
end
